@@ -44,7 +44,6 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#include <sys/mman.h>
 #include <sys/stat.h>
 #include <stdio.h>
 #include <sys/types.h>
@@ -75,9 +74,9 @@ ipod_gstring_flush (struct iPodSharedDataBuffer *shared, GError **error)
 {
 	gboolean success;
 
-	success = itdb_file_set_contents (shared->filename, 
-	 	 		          shared->data->str, shared->data->len,
-				          error);
+	success = g_file_set_contents (shared->filename, 
+	 	 		       shared->data->str, shared->data->len,
+				       error);
 	if (!success) {
                 return FALSE;
 	}
@@ -669,7 +668,6 @@ write_mhba (Itdb_PhotoAlbum *album, iPodBuffer *buffer)
 	    return -1;
 	}
 	total_bytes += bytes_written;
-	mhba = ipod_buffer_get_pointer (buffer);
 
 	for (it = album->members; it != NULL; it = it->next) {
 	        Itdb_Artwork *photo = it->data;
@@ -1180,7 +1178,6 @@ ipod_write_artwork_db (Itdb_iTunesDB *itdb)
 	        g_free (filename);
 		return -1;
 	}
-	g_free (filename);
 	bytes_written = write_mhfd (&db, buf, id_max);
 
 	/* Refcount of the shared buffer should drop to 0 and this should
@@ -1191,8 +1188,10 @@ ipod_write_artwork_db (Itdb_iTunesDB *itdb)
 	if (bytes_written == -1) {
 		g_print ("Failed to save %s\n", filename);
 		/* FIXME: maybe should unlink the file we may have created */
+		g_free (filename);
 		return -1;
 	}
+	g_free (filename);
 	return 0;
 }
 
@@ -1232,13 +1231,15 @@ ipod_write_photo_db (Itdb_PhotoDB *photodb)
 	 * sync buffered data to disk
 	 */
 	ipod_buffer_destroy (buf);
-        g_free (filename);
 
 	if (bytes_written == -1) {
 		g_print ("Failed to save %s\n", filename);
 		/* FIXME: maybe should unlink the file we may have created */
+	        g_free (filename);
 		return -1;
 	}
+        g_free (filename);
+
 	return 0;
 }
 #else
