@@ -23,7 +23,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: itdb_playlist.c 2159 2008-12-08 01:16:58Z tmzullinger $
+|  $Id$
 */
 
 #include "itdb_private.h"
@@ -71,11 +71,14 @@ gboolean itdb_spl_action_known (ItdbSPLAction action)
     case ITDB_SPLACTION_IS_NOT:
     case ITDB_SPLACTION_DOES_NOT_CONTAIN:
     case ITDB_SPLACTION_BINARY_AND:
+    case ITDB_SPLACTION_NOT_BINARY_AND:
+    case ITDB_SPLACTION_BINARY_UNKNOWN1:
+    case ITDB_SPLACTION_BINARY_UNKNOWN2:
 	result = TRUE;
     }
     if (result == FALSE)
     {	/* New action! */
-	g_warning (_("Unknown action (%d) in smart playlist will be ignored.\n"), action);
+	g_warning (_("Unknown action (0x%x) in smart playlist will be ignored.\n"), action);
     }
     return result;
 }
@@ -105,6 +108,14 @@ ItdbSPLFieldType itdb_splr_get_field_type (const Itdb_SPLRule *splr)
     case ITDB_SPLFIELD_COMPOSER:
     case ITDB_SPLFIELD_GROUPING:
     case ITDB_SPLFIELD_TVSHOW:
+    case ITDB_SPLFIELD_CATEGORY:
+    case ITDB_SPLFIELD_DESCRIPTION:
+    case ITDB_SPLFIELD_SORT_SONG_NAME:
+    case ITDB_SPLFIELD_SORT_ALBUM:
+    case ITDB_SPLFIELD_SORT_ALBUMARTIST:
+    case ITDB_SPLFIELD_SORT_ARTIST:
+    case ITDB_SPLFIELD_SORT_COMPOSER:
+    case ITDB_SPLFIELD_SORT_TVSHOW:
 	return ITDB_SPLFT_STRING;
     case ITDB_SPLFIELD_BITRATE:
     case ITDB_SPLFIELD_SAMPLE_RATE:
@@ -115,12 +126,15 @@ ItdbSPLFieldType itdb_splr_get_field_type (const Itdb_SPLRule *splr)
     case ITDB_SPLFIELD_DISC_NUMBER:
     case ITDB_SPLFIELD_BPM:
     case ITDB_SPLFIELD_RATING:
+    case ITDB_SPLFIELD_ALBUM_RATING:
     case ITDB_SPLFIELD_TIME: /* time is the length of the track in
 			   milliseconds */
     case ITDB_SPLFIELD_SEASON_NR:
     case ITDB_SPLFIELD_SKIPCOUNT:
+    case ITDB_SPLFIELD_PODCAST:
 	return ITDB_SPLFT_INT;
     case ITDB_SPLFIELD_COMPILATION:
+    case ITDB_SPLFIELD_PURCHASE:
 	return ITDB_SPLFT_BOOLEAN;
     case ITDB_SPLFIELD_DATE_MODIFIED:
     case ITDB_SPLFIELD_DATE_ADDED:
@@ -176,6 +190,9 @@ ItdbSPLActionType itdb_splr_get_action_type (const Itdb_SPLRule *splr)
 	case ITDB_SPLACTION_IS_IN_THE_LAST:
 	case ITDB_SPLACTION_IS_NOT_IN_THE_LAST:
 	case ITDB_SPLACTION_BINARY_AND:
+	case ITDB_SPLACTION_NOT_BINARY_AND:
+	case ITDB_SPLACTION_BINARY_UNKNOWN1:
+	case ITDB_SPLACTION_BINARY_UNKNOWN2:
 	    return ITDB_SPLAT_INVALID;
 	}
 	/* Unknown action type */
@@ -196,6 +213,9 @@ ItdbSPLActionType itdb_splr_get_action_type (const Itdb_SPLRule *splr)
 	case ITDB_SPLACTION_IS_IN_THE_RANGE:
 	    return ITDB_SPLAT_RANGE_INT;
 	case ITDB_SPLACTION_BINARY_AND:
+	case ITDB_SPLACTION_NOT_BINARY_AND:
+	case ITDB_SPLACTION_BINARY_UNKNOWN1:
+	case ITDB_SPLACTION_BINARY_UNKNOWN2:
 	case ITDB_SPLACTION_IS_STRING:
 	case ITDB_SPLACTION_CONTAINS:
 	case ITDB_SPLACTION_STARTS_WITH:
@@ -240,12 +260,19 @@ ItdbSPLActionType itdb_splr_get_action_type (const Itdb_SPLRule *splr)
 	case ITDB_SPLACTION_IS_NOT:
 	case ITDB_SPLACTION_DOES_NOT_CONTAIN:
 	case ITDB_SPLACTION_BINARY_AND:
+	case ITDB_SPLACTION_NOT_BINARY_AND:
+	case ITDB_SPLACTION_BINARY_UNKNOWN1:
+	case ITDB_SPLACTION_BINARY_UNKNOWN2:
 	    return ITDB_SPLAT_INVALID;
 	}
     case ITDB_SPLFT_BINARY_AND:
 	switch ((ItdbSPLAction)splr->action)
 	{
+	case ITDB_SPLACTION_BINARY_UNKNOWN1:
+	case ITDB_SPLACTION_BINARY_UNKNOWN2:
+            return ITDB_SPLAT_UNKNOWN;
 	case ITDB_SPLACTION_BINARY_AND:
+	case ITDB_SPLACTION_NOT_BINARY_AND:
 	    return ITDB_SPLAT_BINARY_AND;
 	case ITDB_SPLACTION_IS_INT:
 	case ITDB_SPLACTION_IS_NOT_INT:
@@ -295,6 +322,9 @@ ItdbSPLActionType itdb_splr_get_action_type (const Itdb_SPLRule *splr)
 	case ITDB_SPLACTION_IS_NOT:
 	case ITDB_SPLACTION_DOES_NOT_CONTAIN:
 	case ITDB_SPLACTION_BINARY_AND:
+	case ITDB_SPLACTION_NOT_BINARY_AND:
+	case ITDB_SPLACTION_BINARY_UNKNOWN1:
+	case ITDB_SPLACTION_BINARY_UNKNOWN2:
 	    return ITDB_SPLAT_INVALID;
 	}
 	/* Unknown action type */
@@ -535,6 +565,8 @@ gboolean itdb_splr_eval (Itdb_SPLRule *splr, Itdb_Track *track)
 	{
 	case ITDB_SPLACTION_BINARY_AND:
 	    return (intcomp & splr->fromvalue)? TRUE:FALSE;
+	case ITDB_SPLACTION_NOT_BINARY_AND:
+	    return (intcomp & splr->fromvalue)? FALSE:TRUE;
 	}
 	return FALSE;
     case ITDB_SPLFT_BOOLEAN:
@@ -1166,6 +1198,9 @@ Itdb_Playlist *itdb_playlist_duplicate (Itdb_Playlist *pl)
     if (pl->userdata && pl->userdata_duplicate)
 	pl_dup->userdata = pl->userdata_duplicate (pl->userdata);
 
+    /* Copy private data too */
+    pl_dup->priv = g_memdup (pl->priv, sizeof (Itdb_Playlist_Private));
+
     return pl_dup;
 }
 
@@ -1241,6 +1276,8 @@ Itdb_Playlist *itdb_playlist_new (const gchar *title, gboolean spl)
 	/* add at least one rule */
 	itdb_splr_add_new (pl, 0);
     }
+    pl->priv = g_new0 (Itdb_Playlist_Private, 1);
+
     return pl;
 }
 
@@ -1260,22 +1297,13 @@ void itdb_playlist_free (Itdb_Playlist *pl)
     g_list_free (pl->splrules.rules);
     if (pl->userdata && pl->userdata_destroy)
 	(*pl->userdata_destroy) (pl->userdata);
+
+    g_free (pl->priv);
     g_free (pl);
 }
 
-/**
- * itdb_playlist_add:
- * @itdb:   an #Itdb_iTunesDB
- * @pl:     an #Itdb_Playlist
- * @pos:    position to insert @pl at
- *
- * Adds playlist @pl to the database @itdb at position @pos (-1 for
- * "append to end"). A unique id is created if @pl->id is equal to
- * zero. After calling this function, @itdb manages the memory of @pl,
- * which means you no longer need to explicitly call
- * itdb_playlist_free()
- */
-void itdb_playlist_add (Itdb_iTunesDB *itdb, Itdb_Playlist *pl, gint32 pos)
+static void itdb_playlist_add_internal (Itdb_iTunesDB *itdb, Itdb_Playlist *pl,
+                                        gint32 pos, GList **playlists)
 {
     g_return_if_fail (itdb);
     g_return_if_fail (pl);
@@ -1295,7 +1323,7 @@ void itdb_playlist_add (Itdb_iTunesDB *itdb, Itdb_Playlist *pl, gint32 pos)
 	    /* check if id is really unique (with 100 playlists the
 	     * chance to create a duplicate is 1 in
 	     * 184,467,440,737,095,516.16) */
-	    for (gl=itdb->playlists; id && gl; gl=gl->next)
+	    for (gl=*playlists; id && gl; gl=gl->next)
 	    {
 		Itdb_Playlist *g_pl = gl->data;
 		g_return_if_fail (g_pl);
@@ -1308,7 +1336,30 @@ void itdb_playlist_add (Itdb_iTunesDB *itdb, Itdb_Playlist *pl, gint32 pos)
     if (pl->timestamp == 0)  pl->timestamp = time (NULL);
 
     /* pos == -1 appends at the end of the list */
-    itdb->playlists = g_list_insert (itdb->playlists, pl, pos);
+    *playlists = g_list_insert (*playlists, pl, pos);
+}
+
+void itdb_playlist_add_mhsd5_playlist (Itdb_iTunesDB *itdb, Itdb_Playlist *pl,
+                                       gint32 pos)
+{
+    itdb_playlist_add_internal (itdb, pl, pos, &itdb->priv->mhsd5_playlists);
+}
+
+/**
+ * itdb_playlist_add:
+ * @itdb:   an #Itdb_iTunesDB
+ * @pl:     an #Itdb_Playlist
+ * @pos:    position to insert @pl at
+ *
+ * Adds playlist @pl to the database @itdb at position @pos (-1 for
+ * "append to end"). A unique id is created if @pl->id is equal to
+ * zero. After calling this function, @itdb manages the memory of @pl,
+ * which means you no longer need to explicitly call
+ * itdb_playlist_free()
+ */
+void itdb_playlist_add (Itdb_iTunesDB *itdb, Itdb_Playlist *pl, gint32 pos)
+{
+    itdb_playlist_add_internal (itdb, pl, pos, &itdb->playlists);
 }
 
 /**
@@ -1316,9 +1367,10 @@ void itdb_playlist_add (Itdb_iTunesDB *itdb, Itdb_Playlist *pl, gint32 pos)
  * @pl:     an #Itdb_Playlist
  * @pos:    new position
  *
- * Moves playlist @pl to position @pos
+ * Moves playlist @pl to position @pos. -1 will append to the end of the
+ * list.
  */
-void itdb_playlist_move (Itdb_Playlist *pl, guint32 pos)
+void itdb_playlist_move (Itdb_Playlist *pl, gint32 pos)
 {
     Itdb_iTunesDB *itdb;
 
@@ -1531,6 +1583,35 @@ gboolean itdb_playlist_is_podcasts (Itdb_Playlist *pl)
     return (pl->podcastflag == ITDB_PL_FLAG_PODCASTS);
 }
 
+/**
+ * itdb_playlist_is_audiobooks:
+ * @pl: an #Itdb_Playlist
+ *
+ * Checks if @pl is an audiobook playlist
+ *
+ * Returns: TRUE if all tracks in @pl have a mediatype of ITDB_MEDIATYPE_AUDIOBOOK,
+	    FALSE if there are no tracks or one track is not the correct type
+ *
+ */
+gboolean itdb_playlist_is_audiobooks (Itdb_Playlist *pl)
+{
+  GList *tl;
+  Itdb_Track *track;
+  
+  g_return_val_if_fail (pl, FALSE);
+  g_return_val_if_fail (pl->members, FALSE);
+  for (tl = pl->members; tl; tl = tl->next)
+  {
+    track = tl->data;
+    /* Don't check more tracks than necessary */
+    if (!(track->mediatype & ITDB_MEDIATYPE_AUDIOBOOK))
+    {
+      return FALSE;
+    }
+  }  
+  return TRUE;
+}
+
 /**
  * itdb_playlist_set_mpl:
  * @pl: an #Itdb_Playlist
