@@ -80,6 +80,7 @@ struct _SysInfoIpodProperties {
         gint oem_id;
         gint oem_u;
         gint db_version;
+        int shadowdb_version;
         char *min_build_id;
         char *language;
         gboolean voice_memos_supported;
@@ -107,9 +108,10 @@ struct _SysInfoIpodProperties {
         gint games_platform_id;
         gint games_platform_version;
         gint rental_clock_bias;
+	gboolean sqlite_db;
 };
 
-static gint get_int (GHashTable *dict, const char *key)
+static gint64 get_int64 (GHashTable *dict, const char *key)
 {
         GValue *val;
 
@@ -117,10 +119,10 @@ static gint get_int (GHashTable *dict, const char *key)
         if (val == NULL) {
                 return 0;
         }
-        if (!G_VALUE_HOLDS_INT (val)) {
+        if (!G_VALUE_HOLDS_INT64 (val)) {
                 return 0;
         }
-        return g_value_get_int (val);
+        return g_value_get_int64 (val);
 }
 
 static gdouble get_double (GHashTable *dict, const char *key)
@@ -175,8 +177,8 @@ typedef struct _DictFieldMapping DictFieldMapping;
 static const DictFieldMapping sysinfo_ipod_properties_fields_mapping[] = {
     { "BuildID",                       G_TYPE_STRING,  G_STRUCT_OFFSET (SysInfoIpodProperties, build_id) },
     { "ConnectedBus",                  G_TYPE_STRING,  G_STRUCT_OFFSET (SysInfoIpodProperties,  connected_bus) },
-    { "MaxTransferSpeed",              G_TYPE_INT,     G_STRUCT_OFFSET (SysInfoIpodProperties, max_transfer_speed) },
-    { "FamilyID",                      G_TYPE_INT,     G_STRUCT_OFFSET (SysInfoIpodProperties, family_id) },
+    { "MaxTransferSpeed",              G_TYPE_INT64,   G_STRUCT_OFFSET (SysInfoIpodProperties, max_transfer_speed) },
+    { "FamilyID",                      G_TYPE_INT64,   G_STRUCT_OFFSET (SysInfoIpodProperties, family_id) },
     { "ProductType",                   G_TYPE_STRING,  G_STRUCT_OFFSET (SysInfoIpodProperties, product_type) },
     { "FireWireGUID",                  G_TYPE_STRING,  G_STRUCT_OFFSET (SysInfoIpodProperties, firewire_guid) },
     { "FireWireVersion",               G_TYPE_STRING,  G_STRUCT_OFFSET (SysInfoIpodProperties, firewire_version) },
@@ -184,17 +186,17 @@ static const DictFieldMapping sysinfo_ipod_properties_fields_mapping[] = {
     { "MinITunesVersion",              G_TYPE_STRING,  G_STRUCT_OFFSET (SysInfoIpodProperties, min_itunes_version) },
     { "PlaylistFoldersSupported",      G_TYPE_BOOLEAN, G_STRUCT_OFFSET (SysInfoIpodProperties, playlist_folders_supported) },
     { "SerialNumber",                  G_TYPE_STRING,  G_STRUCT_OFFSET (SysInfoIpodProperties, serial_number) },
-    { "UpdaterFamilyID",               G_TYPE_INT,     G_STRUCT_OFFSET (SysInfoIpodProperties, updater_family_id) },
+    { "UpdaterFamilyID",               G_TYPE_INT64,   G_STRUCT_OFFSET (SysInfoIpodProperties, updater_family_id) },
     { "VisibleBuildID",                G_TYPE_STRING,  G_STRUCT_OFFSET (SysInfoIpodProperties, visible_build_id) },
-    { "OEMID",                         G_TYPE_INT,     G_STRUCT_OFFSET (SysInfoIpodProperties, oem_id) },
-    { "OEMU",                          G_TYPE_INT,     G_STRUCT_OFFSET (SysInfoIpodProperties, oem_u) },
-    { "DBVersion",                     G_TYPE_INT,     G_STRUCT_OFFSET (SysInfoIpodProperties, db_version) },
+    { "OEMID",                         G_TYPE_INT64,   G_STRUCT_OFFSET (SysInfoIpodProperties, oem_id) },
+    { "OEMU",                          G_TYPE_INT64,   G_STRUCT_OFFSET (SysInfoIpodProperties, oem_u) },
+    { "DBVersion",                     G_TYPE_INT64,   G_STRUCT_OFFSET (SysInfoIpodProperties, db_version) },
     { "MinBuildID",                    G_TYPE_STRING,  G_STRUCT_OFFSET (SysInfoIpodProperties, min_build_id) },
     { "Language",                      G_TYPE_STRING,  G_STRUCT_OFFSET (SysInfoIpodProperties, language) },
     { "VoiceMemosSupported",           G_TYPE_BOOLEAN, G_STRUCT_OFFSET (SysInfoIpodProperties, voice_memos_supported) },
-    { "UpdateMethod",                  G_TYPE_INT,     G_STRUCT_OFFSET (SysInfoIpodProperties, update_method) },
-    { "MaxFWBlocks",                   G_TYPE_INT,     G_STRUCT_OFFSET (SysInfoIpodProperties, max_fw_blocks) },
-    { "FWPartSize",                    G_TYPE_INT,     G_STRUCT_OFFSET (SysInfoIpodProperties, fw_part_size) },
+    { "UpdateMethod",                  G_TYPE_INT64,   G_STRUCT_OFFSET (SysInfoIpodProperties, update_method) },
+    { "MaxFWBlocks",                   G_TYPE_INT64,   G_STRUCT_OFFSET (SysInfoIpodProperties, max_fw_blocks) },
+    { "FWPartSize",                    G_TYPE_INT64,   G_STRUCT_OFFSET (SysInfoIpodProperties, fw_part_size) },
     { "AutoRebootAfterFirmwareUpdate", G_TYPE_BOOLEAN, G_STRUCT_OFFSET (SysInfoIpodProperties, auto_reboot_after_firmware_update) },
     { "VolumeFormat",                  G_TYPE_STRING,  G_STRUCT_OFFSET (SysInfoIpodProperties, volume_format) },
     { "ForcedDiskMode",                G_TYPE_BOOLEAN, G_STRUCT_OFFSET (SysInfoIpodProperties, forced_disk_mode) },
@@ -205,38 +207,43 @@ static const DictFieldMapping sysinfo_ipod_properties_fields_mapping[] = {
     { "CanHibernate",                  G_TYPE_BOOLEAN, G_STRUCT_OFFSET (SysInfoIpodProperties, can_hibernate) },
     { "CameWithCD",                    G_TYPE_BOOLEAN, G_STRUCT_OFFSET (SysInfoIpodProperties, came_with_cd) },
     { "SupportsSparseArtwork",         G_TYPE_BOOLEAN, G_STRUCT_OFFSET (SysInfoIpodProperties, supports_sparse_artwork) },
-    { "MaxThumbFileSize",              G_TYPE_INT,     G_STRUCT_OFFSET (SysInfoIpodProperties, max_thumb_file_size) },
-    { "RAM",                           G_TYPE_INT,     G_STRUCT_OFFSET (SysInfoIpodProperties, ram) },
-    { "HotPlugState",                  G_TYPE_INT,     G_STRUCT_OFFSET (SysInfoIpodProperties, hotplug_state) },
-    { "BatteryPollInterval",           G_TYPE_INT,     G_STRUCT_OFFSET (SysInfoIpodProperties, battery_poll_interval) },
+    { "MaxThumbFileSize",              G_TYPE_INT64,   G_STRUCT_OFFSET (SysInfoIpodProperties, max_thumb_file_size) },
+    { "RAM",                           G_TYPE_INT64,   G_STRUCT_OFFSET (SysInfoIpodProperties, ram) },
+    { "HotPlugState",                  G_TYPE_INT64,   G_STRUCT_OFFSET (SysInfoIpodProperties, hotplug_state) },
+    { "BatteryPollInterval",           G_TYPE_INT64,   G_STRUCT_OFFSET (SysInfoIpodProperties, battery_poll_interval) },
     { "SortFieldsSupported",           G_TYPE_BOOLEAN, G_STRUCT_OFFSET (SysInfoIpodProperties, sort_fields_supported) },
     { "vCardWithJPEGSupported",        G_TYPE_BOOLEAN, G_STRUCT_OFFSET (SysInfoIpodProperties, vcard_with_jpeg_supported) },
-    { "MaxFileSizeInGB",               G_TYPE_INT,     G_STRUCT_OFFSET (SysInfoIpodProperties, max_file_size_in_gb) },
-    { "MaxTracks",                     G_TYPE_INT,     G_STRUCT_OFFSET (SysInfoIpodProperties, max_tracks) },
-    { "GamesPlatformID",               G_TYPE_INT,     G_STRUCT_OFFSET (SysInfoIpodProperties, games_platform_id) },
-    { "GamesPlatformVersion",          G_TYPE_INT,     G_STRUCT_OFFSET (SysInfoIpodProperties, games_platform_version) },
-    { "RentalClockBias",               G_TYPE_INT,     G_STRUCT_OFFSET (SysInfoIpodProperties, rental_clock_bias) },
+    { "MaxFileSizeInGB",               G_TYPE_INT64,   G_STRUCT_OFFSET (SysInfoIpodProperties, max_file_size_in_gb) },
+    { "MaxTracks",                     G_TYPE_INT64,   G_STRUCT_OFFSET (SysInfoIpodProperties, max_tracks) },
+    { "GamesPlatformID",               G_TYPE_INT64,   G_STRUCT_OFFSET (SysInfoIpodProperties, games_platform_id) },
+    { "GamesPlatformVersion",          G_TYPE_INT64,   G_STRUCT_OFFSET (SysInfoIpodProperties, games_platform_version) },
+    { "RentalClockBias",               G_TYPE_INT64,   G_STRUCT_OFFSET (SysInfoIpodProperties, rental_clock_bias) },
+    { "SQLiteDB",                      G_TYPE_BOOLEAN, G_STRUCT_OFFSET (SysInfoIpodProperties, sqlite_db) },
+    { "ShadowDBVersion",               G_TYPE_INT64,   G_STRUCT_OFFSET (SysInfoIpodProperties, shadowdb_version) },
     { NULL,                            G_TYPE_NONE,    0 }
 };
 
 static const DictFieldMapping sysinfo_image_format_fields_mapping[] = {
-    { "FormatId",         G_TYPE_INT,     G_STRUCT_OFFSET (Itdb_ArtworkFormat, format_id) },
-    { "DisplayWidth",     G_TYPE_INT,     G_STRUCT_OFFSET (Itdb_ArtworkFormat, display_width) },
-    { "RenderWidth",      G_TYPE_INT,     G_STRUCT_OFFSET (Itdb_ArtworkFormat, width) },
-    { "RenderHeight",     G_TYPE_INT,     G_STRUCT_OFFSET (Itdb_ArtworkFormat, height) },
+    { "FormatId",         G_TYPE_INT64,   G_STRUCT_OFFSET (Itdb_ArtworkFormat, format_id) },
+    { "DisplayWidth",     G_TYPE_INT64,   G_STRUCT_OFFSET (Itdb_ArtworkFormat, display_width) },
+    { "RenderWidth",      G_TYPE_INT64,   G_STRUCT_OFFSET (Itdb_ArtworkFormat, width) },
+    { "RenderHeight",     G_TYPE_INT64,   G_STRUCT_OFFSET (Itdb_ArtworkFormat, height) },
 /*  PixelFormat needs to be converted to ItdbThumbFormat, this is special-cased
  *  in g_value_to_image_format */
 /*  { "PixelFormat",      G_TYPE_STRING,  G_STRUCT_OFFSET (Itdb_ArtworkFormat, format) },*/
     { "Interlaced",       G_TYPE_BOOLEAN, G_STRUCT_OFFSET (Itdb_ArtworkFormat, interlaced) },
     { "Crop",             G_TYPE_BOOLEAN, G_STRUCT_OFFSET (Itdb_ArtworkFormat, crop) },
-    { "AlignRowBytes",    G_TYPE_BOOLEAN, G_STRUCT_OFFSET (Itdb_ArtworkFormat, align_row_bytes) },
-    { "Rotation",         G_TYPE_INT,     G_STRUCT_OFFSET (Itdb_ArtworkFormat, rotation) },
+/* AlignRowBytes is an older version of RowBytesAlignment, it's equivalent
+ * to a value of 4 for RowBytesAlignemnt */
+/*    { "AlignRowBytes",    G_TYPE_BOOLEAN, G_STRUCT_OFFSET (Itdb_ArtworkFormat, align_row_bytes) },*/
+    { "Rotation",         G_TYPE_INT64,   G_STRUCT_OFFSET (Itdb_ArtworkFormat, rotation) },
 /* BackColor needs to be converted to a gint, this is special-cased
  * in g_value_to_image_format */
-/*    { "BackColor",        G_TYPE_INT,     G_STRUCT_OFFSET (Itdb_ArtworkFormat, back_color) }, */
-    { "ColorAdjustment",  G_TYPE_INT,     G_STRUCT_OFFSET (Itdb_ArtworkFormat, color_adjustment) },
+/*    { "BackColor",        G_TYPE_INT64   G_STRUCT_OFFSET (Itdb_ArtworkFormat, back_color) }, */
+    { "ColorAdjustment",  G_TYPE_INT64,   G_STRUCT_OFFSET (Itdb_ArtworkFormat, color_adjustment) },
     { "GammaAdjustment",  G_TYPE_DOUBLE,  G_STRUCT_OFFSET (Itdb_ArtworkFormat, gamma) },
-    { "AssociatedFormat", G_TYPE_INT,     G_STRUCT_OFFSET (Itdb_ArtworkFormat, associated_format) },
+    { "AssociatedFormat", G_TYPE_INT64,   G_STRUCT_OFFSET (Itdb_ArtworkFormat, associated_format) },
+    { "RowBytesAlignment", G_TYPE_INT64,   G_STRUCT_OFFSET (Itdb_ArtworkFormat, row_bytes_alignment) },
     { NULL,               G_TYPE_NONE,    0 }
 };
 
@@ -255,10 +262,10 @@ static void dict_to_struct (GHashTable *dict,
     g_return_if_fail (it != NULL);
     while (it->name != NULL) {
         switch (it->type) {
-            case G_TYPE_INT: {
+            case G_TYPE_INT64: {
                 gint *field;
                 field = G_STRUCT_MEMBER_P (struct_ptr, it->offset);
-                *field = get_int (dict, it->name);
+                *field = get_int64 (dict, it->name);
                 break;
             }
 
@@ -333,7 +340,7 @@ static void dump_struct (const DictFieldMapping *mapping,
     g_return_if_fail (it != NULL);
     while (it->name != NULL) {
         switch (it->type) {
-            case G_TYPE_INT: {
+            case G_TYPE_INT64: {
                 gint *field;
                 field = G_STRUCT_MEMBER_P (struct_ptr, it->offset);
                 g_print ("%s: %d\n", it->name, *field);
@@ -368,6 +375,7 @@ static void dump_struct (const DictFieldMapping *mapping,
 static void dump_image_format (Itdb_ArtworkFormat *format)
 {
     dump_struct (sysinfo_image_format_fields_mapping, format);
+    g_print ("PixelFormat: %d\n", format->format); 
 }
 
 void itdb_sysinfo_properties_dump (SysInfoIpodProperties *props)
@@ -396,11 +404,18 @@ set_pixel_format (Itdb_ArtworkFormat *img_spec, GHashTable *dict)
         img_spec->format = THUMB_FORMAT_RGB565_LE;
     } else if (strcmp (pixel_format, "79343230" /* y420 */) == 0) {
         img_spec->format = THUMB_FORMAT_I420_LE;
+    } else if (strcmp (pixel_format, "4C353535" /* L555 */) == 0) {
+	if (g_hash_table_lookup (dict, "PixelOrder") != NULL) {
+	    img_spec->format = THUMB_FORMAT_REC_RGB555_LE;
+	} else {
+	    img_spec->format = THUMB_FORMAT_RGB555_LE;
+	}
     } else {
         g_free (pixel_format);
         return FALSE;
     }
     g_hash_table_remove (dict, "PixelFormat");
+    g_hash_table_remove (dict, "PixelOrder");
     g_free (pixel_format);
     return TRUE;
 }
@@ -447,6 +462,14 @@ static Itdb_ArtworkFormat *g_value_to_image_format (GValue *value)
 
     dict_to_struct (dict, sysinfo_image_format_fields_mapping, img_spec);
 
+    if (get_boolean (dict, "AlignRowBytes")
+            && (img_spec->row_bytes_alignment == 0)) {
+        /* at least the nano3g has the AlignRowBytes key with no
+         * RowBytesAlignment key.
+         */
+        img_spec->row_bytes_alignment = 4;
+    }
+
     return img_spec;
 }
 
@@ -468,6 +491,12 @@ static GList *parse_one_formats_list (GHashTable *sysinfo_dict,
     array = (GValueArray*)g_value_get_boxed (to_parse);
     for (i = 0; i < array->n_values; i++) {
         Itdb_ArtworkFormat *format;
+	/* SysInfoExtended on the iPhone has <string> fields in the artwork
+	 * format array in addition to the hash we parse
+	 */
+	if (!G_VALUE_HOLDS (g_value_array_get_nth (array, i), G_TYPE_HASH_TABLE)) {
+	    continue;
+	}
 	format = g_value_to_image_format (g_value_array_get_nth (array, i));
 	if (format != NULL) {
 		formats = g_list_prepend (formats, format);
@@ -532,6 +561,25 @@ SysInfoIpodProperties *itdb_sysinfo_extended_parse (const char *filename,
     return props;
 }
 
+SysInfoIpodProperties *itdb_sysinfo_extended_parse_from_xml (const char *xml,
+							     GError **error)
+{
+    GValue *parsed_doc;
+    SysInfoIpodProperties *props;
+
+    g_return_val_if_fail (xml != NULL, NULL);
+
+    parsed_doc = itdb_plist_parse_from_memory (xml, strlen (xml), error);
+    if (parsed_doc == NULL) {
+        return NULL;
+    }
+    props = g_value_to_ipod_properties (parsed_doc);
+    g_value_unset (parsed_doc);
+    g_free (parsed_doc);
+
+    return props;
+}
+
 /**
  * itdb_sysinfo_properties_get_serial_number:
  * @props: a #SysInfoIpodProperties structure
@@ -632,7 +680,7 @@ itdb_sysinfo_properties_get_chapter_image_formats (const SysInfoIpodProperties *
  * Returns: TRUE if the iPod supports sparse artwork, FALSE if it does not
  * or if @props doesn't contain any information about sparse artwork
  */
-G_GNUC_INTERNAL gboolean
+gboolean
 itdb_sysinfo_properties_supports_sparse_artwork (const SysInfoIpodProperties *props)
 {
     g_return_val_if_fail (props != NULL, FALSE);
@@ -647,3 +695,40 @@ itdb_sysinfo_properties_supports_podcast (const SysInfoIpodProperties *props)
 
     return props->podcasts_supported;
 }
+
+const char *
+itdb_sysinfo_properties_get_firmware_version (const SysInfoIpodProperties *props)
+{
+    g_return_val_if_fail (props != NULL, NULL);
+
+    return props->visible_build_id;
+}
+
+gboolean
+itdb_sysinfo_properties_supports_sqlite (const SysInfoIpodProperties *props)
+{
+    g_return_val_if_fail (props != NULL, FALSE);
+
+    return props->sqlite_db;
+}
+
+gint
+itdb_sysinfo_properties_get_family_id (const SysInfoIpodProperties *props)
+{
+    g_return_val_if_fail (props != NULL, FALSE);
+    return props->family_id;
+}
+
+gint
+itdb_sysinfo_properties_get_db_version (const SysInfoIpodProperties *props)
+{
+    g_return_val_if_fail (props != NULL, FALSE);
+    return props->db_version;
+}
+
+gint
+itdb_sysinfo_properties_get_shadow_db_version (const SysInfoIpodProperties *props)
+{
+   g_return_val_if_fail (props != NULL, 0);
+   return props->shadowdb_version;
+}
