@@ -23,14 +23,13 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: itdb_track.c 2159 2008-12-08 01:16:58Z tmzullinger $
+|  $Id$
 */
 
 #include <config.h>
 
 #include "itdb_private.h"
 #include "itdb_device.h"
-#include "itdb_thumb.h"
 #include <string.h>
 
 /**
@@ -47,6 +46,7 @@ Itdb_Track *itdb_track_new (void)
 
     track->artwork = itdb_artwork_new ();
     track->chapterdata = itdb_chapterdata_new ();
+    track->priv = g_new0 (Itdb_Track_Private, 1);
 
     track->visible = 1;
     return track;
@@ -71,6 +71,7 @@ static void itdb_track_set_defaults (Itdb_Track *tr)
     gchar *audible_subdesc[] = {"Audible", "audible", "Book", "book", NULL};
     gchar *wav_desc[] = {"WAV", "wav", NULL};
     gchar *m4v_desc[] = {"M4V", "MP4", "MP4V", "m4v", "mp4", "mp4v", NULL};
+    gchar *mov_desc[] = {"MOV", "mov", NULL};
 
     g_return_if_fail (tr);
     g_return_if_fail (tr->itdb);
@@ -153,7 +154,8 @@ static void itdb_track_set_defaults (Itdb_Track *tr)
 	 */
 	if (tr->mediatype == 0)
 	{
-	    if (haystack (tr->filetype, m4v_desc))
+	    if (haystack (tr->filetype, m4v_desc)
+		 || haystack (tr->filetype, mov_desc))
 	    {
 		/* set type to video (0x00000002) */
 		tr->mediatype = 0x00000002;
@@ -260,6 +262,7 @@ void itdb_track_free (Itdb_Track *track)
     if (track->userdata && track->userdata_destroy)
 	(*track->userdata_destroy) (track->userdata);
 
+    g_free (track->priv);
     g_free (track);
 }
 
@@ -351,6 +354,8 @@ Itdb_Track *itdb_track_duplicate (Itdb_Track *tr)
     tr_dup->sort_composer = g_strdup (tr->sort_composer);
     tr_dup->sort_tvshow = g_strdup (tr->sort_tvshow);
 
+    /* Copy private data too */
+    tr_dup->priv = g_memdup (tr->priv, sizeof (Itdb_Track_Private));
 
     /* Copy chapterdata */
     tr_dup->chapterdata = itdb_chapterdata_duplicate (tr->chapterdata);
